//  **** BEGIN LICENSE BLOCK ****
//  Copyright(c) 2005-2006 Adam Judson
//
//
//  PhotoMap
//  - display user contributed photos in a map
//
//  This program is free software; you can redistribute it and/or modify it under
//  the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
//
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
//  more details.
//
//  You should have received a copy of the GNU General Public License along with
//  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
//  Place, Suite 330, Boston, MA 02111-1307 USA
//
//  **** END LICENSE BLOCK ****


// the wrapper for our specific map implementation
function PhotoMapGMap() {
   this.init();
}

PhotoMapGMap.prototype = {
   __proto__ : new PhotoMapBaseMap(),

   zoomLevels : 16,

   getIncludes : function(preferences) {
      return '<script src="http://maps.google.com/maps?file=api&v=1&key=' + 
            preferences.getGoogleKey() +
            '" type="text/javascript"></script>\n';
   },

   getOnload : function(homeLat, homeLon, homeZoom) {
      var onloadText =  '    function onLoad() {\n' +
                        '      document.getElementById("mapContainer").controller = photoMapController;\n' +
                        '      \n' +
                        '      var photoMap = new GMap(document.getElementById("mapContainer"));\n' +
                        '      photoMap.centerAndZoom(new GPoint(' + homeLon + ', ' + homeLat + '), ' + homeZoom + ');\n' +
                        '      photoMap.addControl(new GLargeMapControl());\n' +
                        '      photoMap.addControl(new GMapTypeControl());\n' +
                        '      photoMapController.initMap(photoMap);\n' +
                        '    }\n';
      return onloadText;
   },

   getController : function() {
      var controllerText =    '    function PhotoMapController() {\n' +
                              '      \n' +
                              '      this.initMap = function(photoMap) {\n' +
                              '         this.map = photoMap;\n' +
                              '         this.markers = {};\n' +
                              '      };\n' +
                              '      \n' +
                              '      \n' +
                              '      this.centerAndZoom = function(lat, lon, zoom) {\n' +
                              '         this.map.centerAndZoom(new GPoint(lon, lat), zoom);\n' +
                              '      };\n' +
                              '      \n' +
                              '      this.getLocation = function() {\n' +
                              '         var centre = this.map.getCenterLatLng();\n' +
                              '         return {lat : centre.y, lon : centre.x};\n' +
                              '      };\n' +
                              '      \n' +
                              '      this.getBounds = function() {\n' +
                              '         var bounds = this.map.getBoundsLatLng();\n' +
                              '         return {minX : bounds.minX, minY : bounds.minY, maxX : bounds.maxX, maxY : bounds.maxY};\n' +
                              '      };\n' +
                              '      \n' +
                              '      this.getZoom = function() {\n' +
                              '         return this.map.getZoomLevel();\n' +
                              '      };\n' +
                              '      \n' +
                              '      this.setHandler = function(handler) {\n' +
                              '         this.handler = handler;\n' +
                              '         GEvent.addListener(this.map, "moveend", photoMapMoveEnd);\n' +
                              '      };\n' +
                              '      \n' +
                              '      this.moveEnd = function() {\n' +
                              '        this.handler.mapMoved(this.getLocation(), this.getBounds());\n' +
                              '      }\n' +
                              '      \n' +
                              '      this.setMapTypeMap = function() {\n' +
                              '        this.map.setMapType(G_MAP_TYPE);\n' +
                              '      }\n' +
                              '      \n' +
                              '      this.setMapTypeSatellite = function() {\n' +
                              '        this.map.setMapType(G_SATELLITE_TYPE);\n' +
                              '      }\n' +
                              '      \n' +
                              '      this.setMapTypeHybrid = function() {\n' +
                              '        this.map.setMapType(G_HYBRID_TYPE);\n' +
                              '      }\n' +
                              '      \n' +
                              '      this.getType = function() {\n' +
                              '        if (this.map.getCurrentMapType() == G_HYBRID_TYPE) {\n' +
                              '           return "hybrid";\n' +
                              '        } else if (this.map.getCurrentMapType() == G_SATELLITE_TYPE) {\n' +
                              '           return "satellite";\n' +
                              '        } else {\n' +
                              '           return "map";\n' +
                              '        }\n' +
                              '      }\n' +
                              '      \n' +
                              '      this.markerSelected = function(marker, imgObj) {\n' +
                              '        marker.openInfoWindowHtml("<div style=\'white-space:nowrap;\' ><a href=\'\' onclick=\'window.open(\\"" + imgObj.url + "\\");return false;\'>You are here:<br><img width=\'75px\' height=\'75px\' src=\'" + imgObj.smallUrl + "\'/></a></div>");\n' + 
                              '        this.handler.markerSelected(imgObj.lat, imgObj.lon);' +
                              '      }\n' +
                              '      \n' +
                              '      this.infoWindowOpen = function(marker) {\n' +
                              '        marker.pmOpen = true;' +
                              '      }\n' +
                              '      \n' +
                              '      this.infoWindowClose = function(marker) {\n' +
                              '        if (marker && marker.pmOpen) {\n' +
                              '           delete marker.pmOpen;' +
                              '        }\n' +
                              '      }\n' +
                              '      \n' +
                              '      this.clearMarkers = function() {\n' +
                              '        this.map.clearOverlays();\n' +
                              '        this.markers = {};\n' +
                              '      }\n' +
                              '      \n' +
                              '      this.addMarker = function(imgObj) {\n' +
                              '        if (imgObj) {\n' +
                              '           if (!this.markers[imgObj.key()]) {\n' +
                              '              var point = new GPoint(imgObj.lon, imgObj.lat);\n' +
                              '              var marker = new GMarker(point);\n' +
                              '              GEvent.addListener(marker, "click", function() {photoMapController.markerSelected(marker, imgObj);});\n' +
                              '              GEvent.addListener(marker, "infowindowopen",  function() {photoMapController.infoWindowOpen(marker);});\n' +
                              '              GEvent.addListener(marker, "infowindowclose", function() {photoMapController.infoWindowClose(marker);});\n' +
                              '              this.map.addOverlay(marker);\n' +
                              '              this.markers[imgObj.key()] = marker;\n' +
                              '           }\n' +
                              '        }\n' +
                              '      }\n' +
                              '      \n' +
                              '      this.removeMarker = function(imgObj) {\n' +
                              '        if (imgObj) {\n' +
                              '           var marker = this.markers[imgObj.key()];\n' +
                              '           if (marker) {\n' +
                              '              this.map.removeOverlay(marker);\n' +
                              '              if (marker.pmOpen) {\n' +
                              '                 // window.alert("closing info window");\n' +
                              '                 this.map.closeInfoWindow();\n' +
                              '                 // window.alert("deleted");\n' +
                              '              }\n' +
                              '              delete this.markers[imgObj.key()];\n' +
                              '              // window.alert("deleted");\n' +
                              '           }\n' +
                              '        }\n' +
                              '      }\n' +
                              '      \n' +
                              '      this.addCenterMarker = function() {\n' +
                              '        var loc = this.getLocation();\n' +
                              '        var point = new GPoint(loc.lon, loc.lat);\n' +
                              '        var marker = new GMarker(point);\n' +
                              '        GEvent.addListener(marker, "click", function() {marker.openInfoWindowHtml("<div style=\'white-space:nowrap;\' >This is where the picture will be added.<br>Make sure you are at the right <br>zoom level.</div>");});\n' +
                              '        this.map.addOverlay(marker);\n' +
                              '      }\n' +
                              '      \n' +
                              '    }\n' +
                              '\n' +
                              'photoMapController = new PhotoMapController();\n' + 
                              '\n' +
                              'function photoMapMoveEnd() {\n' +
                              '   photoMapController.moveEnd();\n' +
                              '};\n' +
                              '\n';

      return controllerText;
   },

   ensureInit : function(mapFrame) {
      if (!this.mapController) {
         this.mapController = mapFrame.contentWindow.document.getElementById("mapContainer").wrappedJSObject.controller;
         this.mapController.setHandler(this);
      }
   },

   setLocation : function(lat, lon, zoom) {
      this.mapController.centerAndZoom(lat, lon, this.convertToMapZoom(zoom));
   },

   getLocation : function() {
      return this.mapController.getLocation();
   },

   getBounds : function() {
      return this.mapController.getBounds();
   },

   convertToMapZoom : function(pmZoom) {
      return pmZoom - 1;
   },

   // returns zoom level from 1 - 17
   // where at level 17 we see the whole world
   getZoom : function() {
      return this.mapController.getZoom() + 1;
   },

   addMarker : function(imgObj) {
      this.mapController.addMarker(imgObj);
   },

   removeMarker : function(imgObj) {
      this.mapController.removeMarker(imgObj);
   },

   clearMarkers : function() {
      this.mapController.clearMarkers();
   },

   addCenterMarker : function() {
      this.mapController.addCenterMarker();
   }
}
