//  **** BEGIN LICENSE BLOCK ****
//  Copyright(c) 2005-2006 Adam Judson
//
//
//  PhotoMap
//  - display user contributed photos in a map
//
//  This program is free software; you can redistribute it and/or modify it under
//  the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
//
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
//  more details.
//
//  You should have received a copy of the GNU General Public License along with
//  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
//  Place, Suite 330, Boston, MA 02111-1307 USA
//
//  **** END LICENSE BLOCK ****


// agent to talk to photo implementation
function PhotoMapPhotoAgent(controller) {
   this.init(controller);
}

PhotoMapPhotoAgent.CACHE_LOCATION = "http://photomap.mozdev.org/notTheCache.html";

PhotoMapPhotoAgent.prototype = {
   preferences : new PhotoMapPreferences(),

   init : function(controller) {
      this.controller = controller;
      this.type = this.preferences.getPhotoServiceType();
      this.iPhotoService = this.createPhotoServiceImplementation();
      this.photosByTag = {};
      this.photos = {};
      this.getStoredSearches();
      this.searchQueue = new TimedQueue(400, 3, new Callback(this, this.internalSearch), new Callback(this, this.canDoSearches));
      if (this.iPhotoService) {
         this.iPhotoService.setController(this);
         this.doDefaultSearches();
      }
   },

   shutdown : function() {
      this.saveSearches();
   },

   getStoredSearches : function() {
      var data = PhotoMapUtils.readCacheEntry("HTTP", PhotoMapPhotoAgent.CACHE_LOCATION);
      var searches = this.makeSearches(data);
      this.searches = searches;

      this.makePhotos(data);
   },

   saveSearches : function() {
      var searchData = this.toXML(this.searches);
      PhotoMapUtils.log("Saving searches: [" + searchData.length + "]");
      PhotoMapUtils.writeCacheEntry("HTTP", PhotoMapPhotoAgent.CACHE_LOCATION, searchData);
   },

   toXML : function(searches) {
      var photoKey;
      var xmlString = "<?xml version=\"1.0\"?>\n" +
                      "<photomap>\n" + 
                      "   <searches>\n";
      for (var search in this.searches) {
         xmlString += "<search>\n";
         xmlString += "<key>" + search+ "</key>\n";
         xmlString += "<type>" + this.searches[search].type + "</type>\n";
         xmlString += "<photokeys>";
         for (photoKey in this.searches[search]) {
            if (photoKey != "type") {
               xmlString += "<photokey>" + photoKey + "</photokey>";
            }
         }
         xmlString += "</photokeys>";
         xmlString += "</search>\n";
      }
      xmlString += "</searches>\n";

      xmlString += "<photos>\n";
      for (photoKey in this.photos) {
         xmlString += this.photos[photoKey].toXML();
      }
      xmlString += "</photos>";
      xmlString += "</photomap>";
      return xmlString;
   },

   makeSearches : function(xmlString) {
      var tmpSearches = {};
      try {
         var parser = new DOMParser();
         var domTree = parser.parseFromString(xmlString, "text/xml");
         var searches = document.evaluate("//search", domTree, null, XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);   
         var currentSearch = searches.iterateNext();
         var photos, photo, photoNode, search, key;
         while (currentSearch) {
            key = FlickrPhoto.getStringValue("key", currentSearch);
            search = new Array();
            search.type = FlickrPhoto.getStringValue("type", currentSearch);
            photos = document.evaluate("photokeys/photo", currentSearch, null, XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);   
            photo = photos.iterateNext();
            while (photo) {
               search[photo.stringValue] = "a";
               photo = photos.iterateNext();
            }
            tmpSearches[key] = search;
            currentSearch = searches.iterateNext();
         }
      } catch (e) {
         PhotoMapUtils.log("There was an exception in PhotoMapPhotoAgent.makeSearches: " + e);
      }
      return tmpSearches;
   },

   makePhotos : function(xmlString) {
      var parser = new DOMParser();
      var domTree = parser.parseFromString(xmlString, "text/xml");
      var photos = document.evaluate("//photo", domTree, null, XPathResult.ORDERED_NODE_ITERATOR_TYPE, null);   
      var photoNode = photos.iterateNext();
      while (photoNode) {
         photo = FlickrPhoto.fromXML(photoNode);
         this.storePhoto(photo);
         photoNode = photos.iterateNext();
      }
   },


   // not really happy with this factory, but it seems
   // like a waste to make it more complicated, as all the
   // implementations need to be included in the .xul file anyway
   createPhotoServiceImplementation : function() {
      var iPhotoService;
      if (this.type == "flickr") { 
         iPhotoService = new PhotoMapFPhoto();
      } else {
         window.alert("Photo type not implemented [" + this.type + "]");
      }
      return iPhotoService;
   },

   // return a different one if we aren't flickr
   getPhotoClass : function() {
      return FlickrPhoto;
   },


   setupPhotoService : function() {
      if (this.iPhotoService) {
         // maybe we need an init call here
      }
   },
   
   doDefaultSearches : function() {
      // get all the really important pictures
      // there better not be too many...
      var tags = [];
      tags.type = Photo.WONDER;
      this.searchPhotos(tags);
   },

   locationChanged : function(loc, bounds, zoom, force) {
      if (this.iPhotoService) {
         var searchTags = new SearchTags(bounds, zoom);
         var searches = searchTags.getSearches();
         // if there are search transactions in the queue, drop them, it was probably just
         // a transitional move anyway
         this.searchQueue.clear();
         for (var currentSearch = 0; currentSearch < searches.length; currentSearch++) {
            this.searchPhotos(searches[currentSearch], force);
         }
      } else {
         window.alert("No iPhotoService");
      }
   },

   searchPhotos : function(searchTags, force) {
      var key = searchTags.join();
      var type = searchTags.type;
      if (!force && this.alreadyDoneSearch(key, type)) {
         // we found it, don't do it again
         // but we still need to update the controller
         var searchResults = this.getSearchResults(key);
         if (searchResults) {
            var photo;
            for (var photoKey in searchResults) {
               // using an object as a hash table, then adding other things (type) to it.  Very bad
               if (photoKey != "type") {
                  this.controller.markerUpdate(photoKey);
               }
            }
         }
      } else {
         searchTags.push("pmType:" + type);
         this.searchQueue.add({tags : searchTags, key: key, type : type, forced : force});
      }
   },

/*
   getCurrentResults : function(loc, bounds, zoom) {
      var photoKeys = new Array();
      var searchTags = new SearchTags(bounds, zoom);
      var searches = searchTags.getSearches();
      var searchResults, photoKey;
      var key= "";
      for (var currentSearch = 0; currentSearch < searches.length; currentSearch++) {
         key = searches[currentSearch].join();
         searchResults = this.getSearchResults(key);
         if (searchResults) {
            for (photoKey in searchResults) {
               // using an object as a hash table, then adding other things (type) to it.  Very bad
               if (photoKey != "type") {
                  photoKeys.push(photoKey);
               }
            }
         }
      }
      return photoKeys;
   },
*/

   getSearchResults : function(key) {
      var results = null;
      if (this.searches && this.searches[key]) {
         results = this.searches[key];
      }
      return results;
   },

   internalSearch : function(searchInfo) {
      this.iPhotoService.searchPhotos(searchInfo.tags, new Callback(this, this.searchUpdate, [searchInfo.key, searchInfo.type, searchInfo.forced]));
   },

   alreadyDoneSearch : function(key, type) {
      if (this.searches && this.searches[key] && this.searches[key].type <= type) {
         return true;
      } else {
         return false;
      }
   },

   // callback from the search transaction
   searchUpdate : function(key, type, force, txnPhotos) {
      var search = {};
      search.type = type;
      for (var photo in txnPhotos) {
         // do we already have it?
         if (!force && this.photos[txnPhotos[photo].url]) {
            search[this.photos[txnPhotos[photo].url].key()] = "a";
         } else {
            // FIXME
            // if we have the photo, but we are in force mode - we should probably
            // trash the old pictures...

            // the photo info transaction will update it for us
            // stash the search in the photo for now
            txnPhotos[photo].search = search;
         }
      }
      this.searches[key] = search;
   },

   stop : function() {
      // called when we are set to edit mode,
      // kill/ignore any pending transactions
      this.searchQueue.clear();
   },

   getTags : function(editInfo) {
      return this.getPhotoClass().getTags(editInfo);
   },

   needsAuthentication : function() {
      return this.iPhotoService.needsAuthentication();
   },

   authenticate : function() {
      this.iPhotoService.authenticate();
   },

   doneAuthenticate : function() {
      this.iPhotoService.doneAuthenticate();
   },

   upload : function(uploadInfo) {
      // should really have a different d.o. for this layer...
      this.iPhotoService.upload(uploadInfo);
   },

   getNewAccountUrl : function() {
      return this.iPhotoService.getNewAccountUrl();
   },

   openWindow : function(url) {
      this.controller.openWindow(url);
   },

   authenticationSuccess : function() {
      this.controller.authenticationSuccess();
   },

   logout : function() {
      this.iPhotoService.logout();
   },

   getUserName : function() {
      return this.iPhotoService.getUserName();
   },

   getTotalPictures : function() {
      return this.iPhotoService.getTotalPictures();
   },

   upload : function(editInfo, callback) {
      this.iPhotoService.upload(editInfo, callback);
   },

   // the service implemenations may have throughput restrictions, 
   // this is a callback for the queue which returns true if we can continue processing
   // search requests
   canDoSearches : function(queue) {
      return this.iPhotoService.canSendTransactions();
   },

   photoUpdate : function(photo) {
      if (photo) {
         var key = photo.key();
         if (photo.search) {
            photo.search[key] = "a";
            delete photo.search;
         }
         key = this.storePhoto(photo);
         this.controller.markerUpdate(key);
      }
   },

   getByKey : function(key) {
      return this.photosByTag[key];
   },

   getByLatLon : function(lat, lon) {
      return this.getByKey(Photo.key(lat, lon));
   },

   getAll : function() {
      return this.photosByTag;
   },

   storePhoto: function(photo) {
      if (this.photos[photo.url]) {
         // we have it already
         photo = this.photos[photo.url];
      }
      var key = photo.key();
      var photos = this.photosByTag[key];
      if (!photos) {
         photos = {};
         this.photosByTag[key] = photos;
      }
      photos[photo.url] = photo;
      this.photos[photo.url] = photo;
      return key;
   },

   photo : function(photoKey) {
      return this.photos[photoKey];
   }
}
